package com.ehsure.database_viewer;

import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.os.Environment;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;

import androidx.annotation.NonNull;
import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;
import java.util.List;

public class MainActivity extends AppCompatActivity {

    private CustomHorizontalScrollView horScrollview;
    private final List<Entity> mEntities = new ArrayList<>();
    private final List<String> topTabs = new ArrayList<>();
    private SQLiteDatabase sqLiteDatabase;
    private String tableName;
    private int pkColumnIndex;
    private String pkColumnName;
    private CheckBox cbSelectAll;
    private ContentAdapter contentAdapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        tableName = getIntent().getStringExtra("tableName");
        ActionBar ab = getSupportActionBar();
        assert ab != null;
        ab.setTitle(tableName);
        ab.setDisplayHomeAsUpEnabled(true);
        ab.setDisplayShowTitleEnabled(true);
        RecyclerView rvTabRight = findViewById(R.id.rv_tab_right);
        horScrollview = findViewById(R.id.hor_scrollview);
        RecyclerView recyclerContent = findViewById(R.id.recycler_content);
        cbSelectAll = findViewById(R.id.cb_select_all);
        cbSelectAll.setOnCheckedChangeListener((buttonView, isChecked) -> {
            if (isChecked) {
                for (Entity entity : mEntities) {
                    entity.setSelected(isChecked);
                }
            } else {
                boolean isAllChecked = true;
                for (Entity entity : mEntities) {
                    if (!entity.isSelected()) {
                        isAllChecked = false;
                        break;
                    }
                }
                if (isAllChecked) {
                    for (Entity entity : mEntities) {
                        entity.setSelected(false);
                    }
                }
            }
            contentAdapter.notifyDataSetChanged();
        });
        //处理顶部标题部分
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(this);
        linearLayoutManager.setOrientation(LinearLayoutManager.HORIZONTAL);
        rvTabRight.setLayoutManager(linearLayoutManager);
        TopTabAdapter topTabAdapter = new TopTabAdapter(this);
        rvTabRight.setAdapter(topTabAdapter);
        sqLiteDatabase = openDatabase();
        Cursor cursor = sqLiteDatabase.rawQuery("pragma table_info([" + tableName + "])", null);
        while (cursor.moveToNext()) {
            String[] columnNames = cursor.getColumnNames();
            for(String name : columnNames){
                if (TextUtils.equals(name,"name")) {
                    int pk = cursor.getInt(cursor.getColumnIndex("pk"));
                    String columnName = cursor.getString(cursor.getColumnIndex(name));
                    topTabs.add(columnName);
                    if (pk == 1) {
                        pkColumnName = columnName;
                        pkColumnIndex = topTabs.size() - 1;
                    }
                }
            }
        }
        cursor.close();
        topTabAdapter.setDatas(topTabs);
        //处理内容部分
        recyclerContent.setLayoutManager(new LinearLayoutManager(this));
        recyclerContent.setHasFixedSize(true);
        contentAdapter = new ContentAdapter();
        recyclerContent.setAdapter(contentAdapter);
        contentAdapter.setOnContentScrollListener(event -> {
            //处理单个item滚动时,顶部tab需要联动
            if (null != horScrollview) horScrollview.onTouchEvent(event);
        });
        //滚动RV时,同步所有横向位移的item
        recyclerContent.addOnScrollListener(new RecyclerView.OnScrollListener() {
            @Override
            public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
                super.onScrolled(recyclerView, dx, dy);
                List<ContentAdapter.ItemViewHolder> viewHolderCacheList = contentAdapter.getViewHolderCacheList();
                if (null != viewHolderCacheList) {
                    int size = viewHolderCacheList.size();
                    for (int i = 0; i < size; i++) {
                        viewHolderCacheList.get(i).horItemScrollview.scrollTo(contentAdapter.getOffsetX(), 0);
                    }
                }

            }
        });

        //同步顶部tab的横向scroll和内容页面的横向滚动
        //同步滚动顶部tab和内容
        horScrollview.setOnCustomScrollChangeListener((listener, scrollX, scrollY, oldScrollX, oldScrollY) -> {
            //代码重复,可以抽取/////
            contentAdapter.offsetX = scrollX;
            List<ContentAdapter.ItemViewHolder> viewHolderCacheList = contentAdapter.getViewHolderCacheList();
            if (null != viewHolderCacheList) {
                int size = viewHolderCacheList.size();
                for (int i = 0; i < size; i++) {
                    viewHolderCacheList.get(i).horItemScrollview.scrollTo(scrollX, 0);
                }
            }
        });
        refreshData();
    }

    private void refreshData(){
        cbSelectAll.setChecked(false);
        mEntities.clear();
        Cursor query = sqLiteDatabase.query(tableName,null,null,null,null,null,null);
        int columnCount = query.getColumnCount();
        while (query.moveToNext()) {
            Entity entity = new Entity();
            List<String> rightMoveDatas = new ArrayList<>();
            for (int i = 0; i < columnCount; i++) {
                rightMoveDatas.add(query.getString(i));
                if (i == pkColumnIndex) {
                    String id = query.getString(i);
                    entity.setId(id);
                }
            }
            entity.setRightDatas(rightMoveDatas);
            mEntities.add(entity);
        }
        query.close();
        contentAdapter.notifyDataSetChanged();
    }

    private SQLiteDatabase openDatabase() {
        String dbDir= "/data"+ Environment.getDataDirectory().getAbsolutePath() + "/" + getPackageName()+"/databases/";
        String dbPath= dbDir + getIntent().getStringExtra("dbName");
        return SQLiteDatabase.openOrCreateDatabase(dbPath, null);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        menu.add(0, 0, 0, "删除").setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == 0) {
            StringBuilder delIds = new StringBuilder();
            for (Entity entity : mEntities) {
                if (entity.isSelected()) {
                    delIds.append(entity.getId()).append(",");
                }
            }
            String delStr = delIds.deleteCharAt(delIds.length() - 1).toString();
            String delSql = "delete from " + tableName + " where " + pkColumnName + " in ('" + delStr + "')";
            sqLiteDatabase.execSQL(delSql);
            refreshData();
        }
        return false;
    }

    @Override
    protected void onDestroy() {
        sqLiteDatabase.close();
        super.onDestroy();
    }

    @Override
    public boolean onSupportNavigateUp() {
        onBackPressed();
        return true;
    }

    public interface OnContentScrollListener {
        void onScroll(MotionEvent event);
    }

    private class ContentAdapter extends RecyclerView.Adapter<ContentAdapter.ItemViewHolder> {

        private final List<ContentAdapter.ItemViewHolder> mViewHolderList = new ArrayList<>();
        public int offsetX = 0;
        private OnContentScrollListener onContentScrollListener;

        public void setOnContentScrollListener(OnContentScrollListener onContentScrollListener) {
            this.onContentScrollListener = onContentScrollListener;
        }

        @NonNull
        @Override
        public ContentAdapter.ItemViewHolder onCreateViewHolder(@NonNull ViewGroup viewGroup, int position) {
            View view = LayoutInflater.from(MainActivity.this).inflate(R.layout.layout_item_content, viewGroup, false);
            return new ContentAdapter.ItemViewHolder(view);
        }

        @Override
        public void onBindViewHolder(@NonNull final ContentAdapter.ItemViewHolder viewHolder, final int position) {
            if (position % 2 == 0) {
                viewHolder.itemView.setBackgroundColor(MainActivity.this.getResources().getColor(android.R.color.darker_gray));
            } else
                viewHolder.itemView.setBackgroundColor(MainActivity.this.getResources().getColor(android.R.color.background_light));
            final Entity entity = mEntities.get(position);
            viewHolder.cbSelected.setOnCheckedChangeListener(null);
            viewHolder.cbSelected.setChecked(entity.isSelected());
            viewHolder.cbSelected.setOnCheckedChangeListener((buttonView, isChecked) -> {
                entity.setSelected(isChecked);
                if (!isChecked) {
                    cbSelectAll.setChecked(false);
                }
                notifyItemChanged(position);
            });
            //右边滑动部分
            LinearLayoutManager linearLayoutManager = new LinearLayoutManager(MainActivity.this);
            linearLayoutManager.setOrientation(LinearLayoutManager.HORIZONTAL);
            viewHolder.rvItemRight.setLayoutManager(linearLayoutManager);
            viewHolder.rvItemRight.setHasFixedSize(true);
            RightScrollAdapter rightScrollAdapter = new RightScrollAdapter(MainActivity.this);
            rightScrollAdapter.setDatas(entity.getRightDatas());
            viewHolder.rvItemRight.setAdapter(rightScrollAdapter);
            //缓存当前holder
            if (!mViewHolderList.contains(viewHolder)) {
                mViewHolderList.add(viewHolder);
            }
            viewHolder.horItemScrollview.setEventListener(event -> {
                if (null != onContentScrollListener) onContentScrollListener.onScroll(event);
            });
            //由于viewHolder的缓存,在1级缓存取出来是2个viewholder,并且不会被重新赋值,所以这里需要处理缓存的viewholder的位移
            viewHolder.horItemScrollview.getViewTreeObserver().addOnGlobalLayoutListener(() -> {
                if (!viewHolder.isLayoutFinish()) {
                    viewHolder.horItemScrollview.scrollTo(offsetX, 0);
                    viewHolder.setLayoutFinish(true);
                }
            });
        }

        @Override
        public int getItemCount() {
            return mEntities.size();
        }

        public List<ContentAdapter.ItemViewHolder> getViewHolderCacheList() {
            return mViewHolderList;
        }

        public int getOffsetX() {
            return offsetX;
        }

        class ItemViewHolder extends RecyclerView.ViewHolder {
            CheckBox cbSelected;
            RecyclerView rvItemRight;
            CustomHorizontalScrollView horItemScrollview;
            private boolean isLayoutFinish;//自定义字段,用于标记layout

            public boolean isLayoutFinish() {
                return isLayoutFinish;
            }

            public void setLayoutFinish(boolean layoutFinish) {
                isLayoutFinish = layoutFinish;
            }

            public ItemViewHolder(@NonNull View itemView) {
                super(itemView);
                cbSelected = itemView.findViewById(R.id.cb_selected);
                rvItemRight = itemView.findViewById(R.id.rv_item_right);
                horItemScrollview = itemView.findViewById(R.id.hor_item_scrollview);
            }

        }
    }

}