package com.ehsure.database_viewer;

import android.content.Intent;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.os.Environment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.DefaultItemAnimator;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;

public class TableListActivity extends AppCompatActivity {

    private final ArrayList<String> tableNames = new ArrayList<>();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.table_list_page);
        ActionBar ab = getSupportActionBar();
        assert ab != null;
        ab.setTitle("表名");
        ab.setDisplayHomeAsUpEnabled(true);
        ab.setDisplayShowTitleEnabled(true);
        RecyclerView recyclerView = findViewById(R.id.recyclerView);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));
        recyclerView.setHasFixedSize(true);
        recyclerView.setItemAnimator(new DefaultItemAnimator());
        ListAdapter listAdapter = new ListAdapter();
        recyclerView.setAdapter(listAdapter);
        SQLiteDatabase db = openDatabase();
        Cursor cursor = db.rawQuery("select name from sqlite_master where type='table' order by name", null);
        while(cursor.moveToNext()){
            tableNames.add(cursor.getString(0));
        }
        cursor.close();
        db.close();
        listAdapter.notifyDataSetChanged();
    }

    class ListAdapter extends RecyclerView.Adapter<ListAdapter.ItemViewHolder> {

        @NonNull
        @Override
        public ItemViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            View view = LayoutInflater.from(TableListActivity.this).inflate(R.layout.list_title_item, parent, false);
            return new ItemViewHolder(view);
        }

        @Override
        public void onBindViewHolder(@NonNull ItemViewHolder holder, int position) {
            if (position % 2 == 0) {
                holder.itemView.setBackgroundColor(getResources().getColor(android.R.color.primary_text_dark));
            } else
                holder.itemView.setBackgroundColor(getResources().getColor(android.R.color.secondary_text_dark));
            holder.itemView.setOnClickListener(v -> {
                Intent intent = new Intent(TableListActivity.this, MainActivity.class);
                String dbName= getIntent().getStringExtra("dbName");
                intent.putExtra("dbName",dbName);
                intent.putExtra("tableName",tableNames.get(position));
                startActivity(intent);
            });
            holder.tvTitle.setText(tableNames.get(position));
        }

        @Override
        public int getItemCount() {
            return tableNames.size();
        }

        class ItemViewHolder extends RecyclerView.ViewHolder {
            TextView tvTitle;
            ItemViewHolder(View view) {
                super(view);
                tvTitle = view.findViewById(R.id.tv_title);
            }
        }
    }

    private SQLiteDatabase openDatabase() {
        String dbDir= "/data"+ Environment.getDataDirectory().getAbsolutePath() + "/" + getPackageName()+"/databases/";
        String dbPath= dbDir + getIntent().getStringExtra("dbName");
        return SQLiteDatabase.openOrCreateDatabase(dbPath, null);
    }

    @Override
    public boolean onSupportNavigateUp() {
        onBackPressed();
        return true;
    }
}
